#  Copyright (c) 2001 Autonomous Zone Industries
#  Copyright (c) 2002-2009 Zooko Wilcox-O'Hearn
#  This file is part of pyutil; see README.txt for licensing terms.

"""
Benchmark a function for its behavior with respect to N...
"""

import hotshot, math, operator, sys, time

if not hasattr(time, "realtime"):
    if sys.platform in ("win32", "cygwin",):
        time.realtime = time.clock
    else:
        time.realtime = time.time

from assertutil import _assert

def makeg(func):
    def blah(n, func=func):
        for i in xrange(n):
            func()
    return blah

def rep_bench(func, n, initfunc=None, MAXREPS=5, MAXTIME=60.0, profile=False, profresults="pyutil-benchutil.prof"):
    """
    Will run the func up to MAXREPS times, but won't start a new run if MAXTIME
    (wall-clock time) has already elapsed (unless MAXTIME is None).
    """
    starttime = time.realtime()
    print "N: %7d," % n,
    sys.stdout.flush()
    scores = []
    while ((len(scores) < MAXREPS) or (MAXREPS is None)) and ((MAXTIME is None) or ((time.realtime() - starttime) < MAXTIME)):
        if initfunc:
            initfunc(n)
        try:
            tl = bench_it(func, n, profile=profile, profresults=profresults)
        except BadMeasure:
            pass
        else:
            scores.append(tl)
    sum = reduce(operator.__add__, scores)
    ave = sum / len(scores)
    scores.sort()
    worst = scores[-1]
    best = scores[0]
    nw = min(int(math.log(len(scores)))+1, len(scores))
    nworst = scores[-nw]
    nbest = scores[nw-1]
    print "time: best: %7.2f, %3dth-best: %7.2f, ave: %7.2f, %3dth-worst: %7.2f, worst: %7.2f (of %6d), reps/s: %s, ave rate: %s" % (best, nw, nbest, ave, nw, nworst, worst, len(scores), (ave > 0) and ("%6d" % (len(scores)/sum)) or "N/A", (ave > 0) and ("%8.0f" % ((n*len(scores))/sum)) or "N/A")
 
MARGINOFERROR = 10

worstemptymeasure = 0

class BadMeasure(Exception):
    """ Either the clock wrapped (which happens with time.clock()) or
    it went backwards (which happens with time.time() on rare
    occasions), (or the code under measure completed before a single
    clock tick, which is probably impossible). """
    pass

def bench_it(func, n, profile=False, profresults="pyutil-benchutil.prof"):
    if profile:
        prof = hotshot.Profile(profresults)
        st = time.realtime()
        prof.runcall(func, n)
        sto = time.realtime()
        prof.close()
    else:
        st = time.realtime()
        func(n)
        sto = time.realtime()
    timeelapsed = sto - st
    if timeelapsed <= 0:
        raise BadMeasure(timeelapsed)
    global worstemptymeasure
    emsta = time.realtime()
    emstop = time.realtime()
    empty = emstop - emsta
    if empty > worstemptymeasure:
        worstemptymeasure = empty
    _assert(timeelapsed*MARGINOFERROR > worstemptymeasure, "Invoking func %s(%s) took only %0.20f seconds, but we cannot accurately measure times much less than %0.20f seconds.  Therefore we cannot produce good results here.  Try benchmarking a more time-consuming variant." % (func, n, timeelapsed, worstemptymeasure,))
    return timeelapsed

def bench(func, initfunc=None, TOPXP=21, MAXREPS=5, MAXTIME=60.0, profile=False, profresults="pyutil-benchutil.prof"):
    BSIZES = []
    for i in range(TOPXP-6, TOPXP+1, 2):
        BSIZES.append(2 ** i)

    for BSIZE in BSIZES:
        rep_bench(func, BSIZE, initfunc=initfunc, MAXREPS=MAXREPS, MAXTIME=MAXTIME, profile=profile, profresults=profresults)
